"""Shared defaults for pyannote diarization/separation models."""

PYANNOTE_DIARIZATION_MODEL_ID = "pyannote/speaker-diarization-community-1"
PYANNOTE_SEPARATION_MODEL_ID = "pyannote/speech-separation-ami-1.0"
